Assets = {
	Asset("ANIM", "anim/eyepos.zip")
}

local function MakeLocations()
    local useablepositions = {}
    for i = 1, 100 do
        local s = i / 32
        local a = math.sqrt(s * 512)
        local b = math.sqrt(s) * 11
        local pos = { x = math.sin(a) * b, y = 0, z = math.cos(a) * b }
        table.insert(useablepositions, pos)
    end
    return useablepositions 
end

local locations = MakeLocations()

local VALID_TILE_TYPES =
{
    [4] = true, -- GROUND.DIRT
    [5] = true, -- GROUND.SAVANNA
    [6] = true, -- GROUND.GRASS
    [7] = true, -- GROUND.FOREST
    [8] = true, -- GROUND.MARSH

    -- CAVES
    [13] = true, -- GROUND.CAVE
    [14] = true, -- GROUND.FUNGUS
    [15] = true, -- GROUND.SINKHOLE
    [17] = true, -- GROUND.MUD
    [24] = true, -- GROUND.FUNGUSRED
    [25] = true, -- GROUND.FUNGUSGREEN

    --EXPANDED FLOOR TILES
    [30] = true, -- GROUND.DECIDUOUS
}

local function CheckTileCompatibility(tile)
    return VALID_TILE_TYPES[tile]
end


local function createhelper(pos,inst)
    local helper = GLOBAL.CreateEntity()
    helper.entity:SetCanSleep(false)
    helper.persists = false

    helper.entity:AddTransform()
    helper.entity:AddAnimState()

    helper:AddTag("CLASSIFIED")
    helper:AddTag("NOCLICK")
    helper:AddTag("placer")

    helper.Transform:SetScale(1.24, 1.24, 1.24)

    helper.AnimState:SetBank("eyepos")
    helper.AnimState:SetBuild("eyepos")
    helper.AnimState:PlayAnimation("idle")
    helper.AnimState:SetLightOverride(1)
    helper.AnimState:SetOrientation(1) -- ANIM_ORIENTATION.OnGround
    helper.AnimState:SetLayer(1) -- LAYER_BACKGROUND
    helper.AnimState:SetSortOrder(1)
    helper.AnimState:SetAddColour(0, .2, .5, 0)

    helper.entity:SetParent(inst.entity)
    helper.Transform:SetPosition(pos.x, 0, pos.z)
	return helper
end

local function OnEnableHelper(inst,enabled)
	if enabled then
		if inst.helpers == nil then
			inst.helpers = {}
			local ground = GLOBAL.TheWorld
			local x,y,z = inst.Transform:GetWorldPosition()
			local currentminions = nil
			if inst.components.minionspawner ~= nil and not inst.components.minionspawner:MaxedMinions() then
				currentminions = inst.components.minionspawner.numminions or 0 
			else return 
			end
			for k,pos in ipairs(locations) do 
				if ground.Map:IsAboveGroundAtPoint(pos.x + x,0,pos.z + z) and
					CheckTileCompatibility(ground.Map:GetTileAtPoint(pos.x + x,0,pos.z + z)) and
					ground.Pathfinder:IsClear(x, 0, z, pos.x + x, 0, pos.z + z, { ignorewalls = true }) and
					#TheSim:FindEntities(pos.x + x, pos.y, pos.z + z, 2.5, { "eyeplant" }) <= 0 and
					#TheSim:FindEntities(pos.x + x, pos.y, pos.z + z, 1) <= 0 and
					not ground.Map:IsPointNearHole(GLOBAL.Vector3(pos.x + x, pos.y, pos.z + z)) then
					table.insert(inst.helpers,createhelper(pos,inst))
					if #inst.helpers >= 32.4 - currentminions then
						break
					end
				end
			end
		end
    elseif inst.helpers ~= nil then
		for k,v in pairs(inst.helpers) do
			inst.helpers[k]:Remove() 
		end
		inst.helpers = nil
    end
end

AddPrefabPostInit("lureplant", function(inst)
    if not GLOBAL.TheNet:IsDedicated() then
        inst:AddComponent("deployhelper")
        inst.components.deployhelper.onenablehelper = OnEnableHelper
    end
end )
